import os
import re
from datetime import datetime

from org_mapper.utils import log_and_sleep

from pscr_common.utils import _logger

prev_run_time = None


def get_query_status(query_result, query):
    query_status = query_result["QueryStatus"]
    progress_percentage = query_status["ProgressPercentage"]
    _logger.debug(
        "Query progress for Query:\n"
        + query
        + "\n so far: "
        + str(progress_percentage)
        + "%"
    )

    data_scanned = float(query_status["CumulativeBytesScanned"]) / 1073741824
    _logger.debug(f"Data scanned so far: {data_scanned} GB")

    data_metered = float(query_status["CumulativeBytesMetered"]) / 1073741824
    _logger.debug(f"Data metered so far: {data_metered} GB")

    return progress_percentage


def build_query(
    identifier_column,
    metric_period,
    env,
    database,
    table,
    start_time,
    end_time,
    metric_list,
):
    metric_stats_str = ""
    for metric_stat in metric_list:
        metric_stats_str += f"{metric_stat[1]}({metric_stat[0]}) as {metric_stat[0]},"

    return f"""
    SELECT {identifier_column}, bin(time, {metric_period}s) AS hour, {metric_stats_str[:-1]}
     FROM \"{env}-{database}\".\"{env}-{table}\"
     WHERE time BETWEEN '{start_time}' and '{end_time}'
     GROUP BY {identifier_column} , bin(time, {metric_period}s) ORDER by 1, 2 ASC
    """


def parse_query_result(query_result, metric_timeseries_data):
    for row in query_result["Rows"]:
        data = row["Data"]
        identifier = data[0]["ScalarValue"]
        time_stamp = datetime.strptime(
            data[1]["ScalarValue"], "%Y-%m-%d %H:%M:%S.%f000"
        )
        data_index = 2
        metric_index = 0

        if bool(re.match("^[^_-]+$", identifier)):
            while data_index < len(data) and metric_index < len(metric_timeseries_data):
                if not data[data_index].get("NullValue"):
                    temp_map = {time_stamp: float(data[data_index]["ScalarValue"])}
                    metric_name = next(iter(metric_timeseries_data[metric_index]))
                    if (
                        metric_timeseries_data[metric_index]
                        .get(metric_name)
                        .get(identifier)
                        is None
                    ):
                        metric_timeseries_data[metric_index].get(metric_name)[
                            identifier
                        ] = []
                    metric_timeseries_data[metric_index].get(metric_name)[
                        identifier
                    ].append(temp_map)

                data_index += 1
                metric_index += 1


def execute_query(paginator, query, metric_timeseries_data):
    retries = 0
    while True:
        try:
            page_iterator = paginator.paginate(QueryString=query)
            for page in page_iterator:
                get_query_status(page, query)
                parse_query_result(page, metric_timeseries_data)
            _logger.debug(f"Success executing query {query}")
        except Exception as ex:
            log_and_sleep(
                f"Timestream Query Exception: {ex}  while executing  query  {query}, "
                f"Retrying",
                retries,
                max_sleep=60,
            )
        else:
            break
        retries += 1

    return metric_timeseries_data
